//******************************************************************************
// Project: Simple LAN Monitor v1.00
// URL:		http://slam.sourceforge.net
// File:    AboutBox.java
//
// Description:
//          Creates and manages an About Dialog Box, with the authors and the
//          program information.
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Panel;
import java.awt.Image;
import java.awt.Button;
import java.awt.TextArea;
import java.awt.SystemColor;
import java.awt.Window;
import java.awt.Frame;
import java.awt.event.*;
import java.util.ResourceBundle;

import InvalidParameter;
import WinUtil;
import LicenceBox;


/**
 * Creates and manages an <i>About Dialog Box</i>, with the authors and the
 * program information.
 * 
 * This class is derived from <b>java.awt.Frame<b>.
 * 
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.0, 2001-10-21
 * @since	1.0
 */
public class AboutBox extends Frame
					  implements WindowListener , ActionListener, FocusListener {
	// Private members
	private static long m_lCount = 0;	// Instances counter
	private static boolean m_bRunning = false;
	
	// Resources
	private ResourceBundle m_ResourcesBundle = ResourceBundle.getBundle("StringResources", getLocale());
	
	// Buttons, TextArea and Icon
	private Button m_btnOK = new Button(m_ResourcesBundle.getString("OKKey"));
	private Button m_btnLicence = new Button(m_ResourcesBundle.getString("LicenceKey"));
	private TextArea m_txtaMessage = new TextArea(m_ResourcesBundle.getString("AboutTextKey"));

	// Constructors
	
	/**
	 * Constructs a About Dialog Box and show it.
	 * 
	 * @param frame	Parent frame. Can be null.
	 * @param title	Title on the Dialog Window.
	 * @param text Message to display.
	 */
	public AboutBox(String title)
	{
		super(title);

		// Only one instance can run at a time
		if (m_bRunning) {
			this.dispose();
			return;
		}
		else {
			m_bRunning = true;
				
			// Give a name to this message box
			setName("aboutbox" + String.valueOf(m_lCount));
			++m_lCount;
		
			DialogInit();
		}
	}
	
	public synchronized boolean isRunning()
	{
		return m_bRunning;	
	}
	
	private void DialogInit()
	{	
		Toolkit.getDefaultToolkit().beep();
		String text = m_txtaMessage.getText();
		
		// Colors
		setBackground((Color)SystemColor.control);
		setForeground((Color)SystemColor.controlText);
		
		// TextArea setup
		m_txtaMessage.addFocusListener(this);
		m_txtaMessage.setEditable(false);

		// Add 'OK' and 'Licence' buttons
		Panel p = new Panel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER, 4, 2));
		m_btnOK.addActionListener(this);
		m_btnOK.setSize(50, 30);
		p.add(m_btnOK);
		m_btnLicence.addActionListener(this);
		m_btnLicence.setSize(50, 30);
		p.add(m_btnLicence);
		
		// Calculates the area of the text
		String strLF = System.getProperty("line.separator");
		int c = 0;
		int l = 0;
		int r1 = 0;
		int r2 = 0;
		int i;
		int j;
		String str;
		if (text != null) {
			do {
				r1 = text.indexOf(strLF, r1);
				if (r1 == -1)
					r1 = text.length();
				++l;
				i = r1 - r2;
				try {
					// If the text contains tabs ('\t'), increments i
					// Tabs is consider to be 8 spaces
					str = text.substring(r2, r1);
					j = 0;
					do {
						j = str.indexOf((int)'\t', j);
						if (j != -1) {
							i += 7;
							++j;
						}
					} while (j != -1);
				}
				catch (StringIndexOutOfBoundsException e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}
				if (c < i)
					c = i;
				r1 = r1 + strLF.length();
				r2 = r1;
			} while (r1 < text.length());
		}
		if (l < 3)
			l = 3;
		if (c < 10)
			c = 10;
		m_txtaMessage.setColumns(c);
		m_txtaMessage.setRows(l);
				
		// Add components to frame window.
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		setLayout(gb);
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbc.gridy = GridBagConstraints.RELATIVE;
		gbc.gridwidth = 1;
		gbc.gridheight = 3;
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.insets = new Insets(4, 4, 4, 4);
		gbc.weightx = 0;
		gbc.weighty = 0;
		
		// Mesage text
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gb.setConstraints(m_txtaMessage, gbc);
		add(m_txtaMessage);

		// Buttons panel
		gbc.gridheight = 1;
				
		gb.setConstraints(p, gbc);
		add(p);

		// Show this dialog
		addWindowListener(this);
		pack();
		setResizable(false);
		try {
			WinUtil.Center(null, this);
		}
		catch(Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		setVisible(true);
	}
	
	// Component methods
	
	public String paramString()
	{
		return new String(getName());
	}

	// ActionListener methods
	
	public void actionPerformed(ActionEvent e)
	{
		Object objSource = e.getSource();
		// Buttons
		if (objSource instanceof Button) {
			Button buttonS = (Button)objSource;
			// OK button
			if (buttonS.equals(m_btnOK)) {
				m_bRunning = false;
				this.dispose();
				return;
			}
			// Licence button
			if (buttonS.equals(m_btnLicence)) {
				m_btnOK.setEnabled(false);
				m_btnLicence.setEnabled(false);
				LicenceBox d = new LicenceBox(this, m_btnLicence.getLabel());
				m_btnOK.setEnabled(true);
				m_btnLicence.setEnabled(true);
				return;
			}
		}
	}
	
	// WindowListener methods
	
	public void windowOpened(WindowEvent e)
	{
	}

	public void windowClosed(WindowEvent e)
	{
	}
	
	public void windowClosing(WindowEvent e)
	{
		if (e.getWindow().equals((Window)this)) {
			m_bRunning = false;
			this.dispose();
		}
	}
	
	public void windowActivated(WindowEvent e)
	{
	}
	
	public void windowDeactivated(WindowEvent e)
	{
	}
	
	public void windowIconified(WindowEvent e)
	{
	}
	
	public void windowDeiconified(WindowEvent e)
	{
	}
	
	// FocusListener methods
	
	public void focusGained(FocusEvent e)
	{
		if (e.getComponent().equals(m_txtaMessage)) {
			m_txtaMessage.transferFocus();
		}	
	}
	
	public void focusLost(FocusEvent e)
	{
	}
}
