//******************************************************************************
// Project: Simple LAN Monitor v1.00
// URL:		http://slam.sourceforge.net
// File:    BorderPanel.java
//
// Description:
//          Creates a panel that as a border. The border can be raised, sunked, 
//          frame unicolor (dark or light), or tick raised or sunked. It can 
//          also have customized colors and tickness.
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.SystemColor;
import java.awt.Graphics;
import java.awt.Container;

import InvalidParameter;
import WinUtil;


/**
 * Creates a panel that as a border. The border can be raised, sunked, frame 
 * unicolor (dark or light),or tick raised or sunked. It can also have, 
 * customized colors and tickness.
 * <p>
 * By default, the system colors for controls hare used and the default 
 * border tickness are:
 * <pre>
 *     - raised and  sunked styles: 2;
 *     - frame style: 4;
 *     - unicolor dark and light styles: 1;
 *     - tick raised and sunked: 4.
 * </pre>
 * 
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.1, 2001-10-21
 * @since	1.0
 */
public class BorderPanel extends Container {
	// Public constants
	
	/**
	 * Raised border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_RAISED	= 0x00000001;
	/**
	 * Sunked border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_SUNKED	= 0x00000002;
	/**
	 * Frame border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_FRAME	= 0x00000003;
	/**
	 * Dark unicolor border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_UNIDARK	= 0x00000004;
	/**
	 * Light unicolor border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_UNILIGHT	= 0x00000005;
	/**
	 * Tick raised border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_TICK_RAISED	= 0x00000006;
	/**
	 * Tick sunked border style flag.
	 * 
	 * @since	1.1
	 */
	public final static int BP_TICK_SUNKED	= 0x00000007;
	
	// Private members
	private static long m_lCount = 0;	// Instances counter
	
	private Image m_offScreen = null;	// Window m_imageOffScreen image buffer
	
	private int m_iStyle;			// Border style (raised or sunked)
	private int m_iSize;			// Border size
	private int m_iBorder;			// Border tickness
	private Color m_ColorDark;		// Border dark color to use
	private Color m_ColorDarkDk;	// Border very dark color to use
	private Color m_ColorLight;		// Border light color to use
	private Color m_ColorLightLt;	// Border very light color to use
	
	// Constructors
	
	/**
	 * Constructs a default raised Border Panel. Uses the system default 
	 * colors for the borders, background and foreground.
	 * <p>
	 * This is the same as <code>BorderPanel(BP_RAISED, 0)</code> constructor.
	 * 
	 * @see		#BorderPanel(int, int)
	 * @since	1.0
	 */
	public BorderPanel() throws InvalidParameter
	{
		this(BP_RAISED, 0);
	}
	
	/**
	 * Constructs a Border Panel with the specified style and its dedault 
	 * tickness. Uses the system default colors for the borders, background 
	 * and forground. The border tickness is set as the default.
	 * <p>
	 * This is the same as <code>BorderPanel(style, size)</code> 
	 * constructor, were <code>size</code> is calculate as following, base on 
	 * <code>style</code>:
	 * <pre>
	 *     - style = BP_TICK_RAISED, BP_TICK_SUNKED or BP_FRAME: 0;
	 *     - style = BP_UNIDARK or BP_UNILIGHT: 1;
	 *     - style = BP_TICK_RAISED or BP_TICK_SUNKED: 4.
	 * </pre>
	 * 
	 * @param	style	The border style for this border.
	 * @see		#BorderPanel(int, int)
	 * @since	1.1
	 */
	public BorderPanel(int style) throws InvalidParameter
	{
		this(style, 1);
		
		int size = 0;
		switch (style) {
			case BP_UNIDARK:
			case BP_UNILIGHT:
				size = 1;
				break;
			case BP_TICK_RAISED:
			case BP_TICK_SUNKED:
				size = 4;
				break;
		}
		this.setBorderSize(size);
	}
	
	/**
	 * Constructs a Border Panel with the specified style and the specified 
	 * tickness. Uses the system default colors for the borders, background 
	 * and forground. The border tickness is calculated based on the size.
	 * <p>
	 * The meaninig for <code>style</code> and <code>size</code>, is as the 
	 * following:
	 * <pre>
	 *     - style = BP_RAISED or BP_SUNKED:
	 *         Draws a raised or sunked border (respectavily). The border 
	 *         tickness is 2 + size, where size is the external gap to the 
	 *         border and can be >= 0;
	 *     - style = BP_FRAME:
	 *         Draws a frame border. The border tickness is 2 + size + 2, 
	 *         where size is the internal gap betwen the outer and iner borders 
	 *         and can be  >= 0;
	 *     - style = BP_UNIDARK or BP_UNILIGHT:
	 *         Draws a unicolor border (a rectangle). The color used for the 
	 *         border is the dark border color or the light border color 
	 *         (respectavily). The border tickness is equal to size and it can 
	 *         be >= 1;
	 *     - style = BP_TICK_RAISED or BP_TICK_SUNKED:
	 *         Draws a tick raised or sunked border (respectavily). The border 
	 *         tickness is equal to size and it can be >= 1;
	 * </pre>
	 * 
	 * @param	style	The border style for this border.
	 * @param	size	Size of the border, as specified above.
	 * @exception InvalidParameter	If an invalid <code>style</code> or 
	 *                              <code>size</code> is passed.
	 * @since	1.0
	 */
	public BorderPanel(int style, int size) throws InvalidParameter
	{
		super();
		
		// Parameters validation
		this.setBorderStyle(style);
		this.setBorderSize(size);
				
		// Variables inicialization
		setBackground((Color)SystemColor.control);
		setForeground((Color)SystemColor.controlText);
		m_ColorDark = (Color)SystemColor.controlShadow;
		m_ColorDarkDk = (Color)SystemColor.controlDkShadow;
		m_ColorLight = (Color)SystemColor.controlLtHighlight;
		m_ColorLightLt = (Color)SystemColor.controlHighlight;
				
		// Give a name to this border panel
		this.setName("borderpanel" + String.valueOf(m_lCount));
		++m_lCount;
	}

	// Container methods 
	
	/**
	 * Returns the insets for this container's border.
	 * 
	 * @see java.awt.Container#getInsets()
	 */
	public Insets getInsets()
	{
		return new Insets(m_iBorder, m_iBorder, m_iBorder, m_iBorder);
	}
	
	/**
	 * Returns the minimum size for this container.
	 * 
	 * @see java.awt.Container#getMinimumSize()
	 */
	public Dimension getMinimumSize()
	{
		int i = (m_iBorder + 1) * 2;
		return new Dimension(i, i);	
	}
	
	/**
	 * Invalidates the container.
	 * <p>
	 * This implementation destroys our bitmap image 
	 * buffer of this panel's window client area.
	 * 
	 * @see java.awt.Container#invalidate()
	 */
	public void invalidate() {
		super.invalidate();
		m_offScreen = null;
	}
	
	/**
	 * Updates the container.
	 * <p>
     * This implementation avoids the background to 
     * be erased by calling <code>paint</code> insted.
	 * 
	 * @param	Graphics object.
	 * @see		java.awt.Container#update(java.awt.Graphics)
	 * @see		#paint(java.awt.Graphics)
	 */
	public void update(Graphics g) {
		paint(g);
	}
	
	/**
	 * Paints the container.
	 * <p>
	 * This implementation paints this panel window 
	 * and all its children at once, using an m_imageOffScreen bitmap image buffer.
	 * <p>
	 * Doing so, avoids flicking of this window and its children.
	 * 
	 * @param	Graphics object.
	 * @see		java.awt.Container#paint(java.awt.Graphics)
	 */
	public void paint(Graphics  g)
	{
		// Variable inicialization
		Rectangle rect = getBounds();
		rect.x = 0;
		rect.y = 0;
		
		// Creates an m_imageOffScreen buffer
		if(m_offScreen == null) {
			m_offScreen = createImage(rect.width, rect.height);
		}
		Graphics og = m_offScreen.getGraphics();
		og.setClip(0, 0, rect.width, rect.height);
		
		// Clear background
		og.setColor(getBackground());
		og.fillRect(0, 0, rect.width, rect.height);
		
		// Draws the border
		switch (m_iStyle) {
			case BP_RAISED:
				WinUtil.Draw3DBorder(og, rect, m_iSize,
							 m_ColorLight, m_ColorDark, 
							 m_ColorLightLt, m_ColorDarkDk);
				break;
			case BP_SUNKED:
				WinUtil.Draw3DBorder(og, rect, m_iSize,
							 m_ColorDarkDk, m_ColorLightLt, 
							 m_ColorDark, m_ColorLight);
				break;
			case BP_FRAME:
				WinUtil.Draw3DFrame(og, rect, m_iSize,
							 m_ColorLight, m_ColorDark, 
							 m_ColorLightLt, m_ColorDarkDk);
				break;
			case BP_UNIDARK:
				WinUtil.DrawBorder(og, rect, m_iSize, m_ColorDark);
				break;
			case BP_UNILIGHT:
				WinUtil.DrawBorder(og, rect, m_iSize, m_ColorLight);
				break;
			case BP_TICK_RAISED:
				WinUtil.Draw3DTickBorder(og, rect, m_iSize,
							 m_ColorLight, m_ColorDark);
				break;
			case BP_TICK_SUNKED:
				WinUtil.Draw3DTickBorder(og, rect, m_iSize,
							 m_ColorDark, m_ColorLight);
				break;
		}
				
		// Paint children
		super.paint(og);
		
		// Blast entire image to screen
		g.drawImage(m_offScreen, 0, 0, null);
		og.dispose();
	}
	
	/**
	 * Returns the parameter string representing the 
	 * state of this container.
	 * 
	 * @see java.awt.Container#paramString()
	 */
	public String paramString()
	{
		return new String(
						  this.getName() + 
						  ", type=" + String.valueOf(this.m_iStyle) + 
						  ", size=" + String.valueOf(this.m_iSize) +
						  ", dark=" + String.valueOf(this.m_ColorDark) +
						  ", light=" + String.valueOf(this.m_ColorLight));
	}
	
	// BorderPanel methods
			
	/**
	 * Gets the border style. Return is: BP_RAISED (raised border) or BP_SUNKED (sunked border).
	 * 
	 * @return The border style (BP_RAISED or BP_SUNKED).
	 * @see #setBorderType(int)
	 */
	public int getBorderStyle()
	{
		return m_iStyle;
	}
	
	/**
	 * Sets the border style. (BP_RAISED = raised border, BP_SUNKED = sunked border).
	 * 
	 * @param style BP_RAISED or BP_SUNKED.
	 * @see #getBorderType()
	 */
	public void setBorderStyle(int style) throws InvalidParameter
	{
		switch (style) {
			case BP_RAISED:
			case BP_SUNKED:
			case BP_FRAME:
			case BP_UNIDARK:
			case BP_UNILIGHT:
			case BP_TICK_RAISED:
			case BP_TICK_SUNKED:
				m_iStyle = style;
				break;
			default:
				throw new InvalidParameter("Invalid border style!");
		}
	}
	
	/**
	 * Gets the border size of this panel.
	 * 
	 * @return The border size.
	 * @see #setBorderSize(int)
	 */
	public int getBorderSize()
	{
		return m_iSize;
	}
	
	/**
	 * Sets the border size of this panel.
	 * 
	 * @param size The border size.
	 * @see #getBorderSize()
	 */
	public void setBorderSize(int size) throws InvalidParameter
	{
		switch (m_iStyle) {
			case BP_RAISED:
			case BP_SUNKED:
			case BP_FRAME:
				if (size >= 0)
					m_iSize = size;
				else
					throw new InvalidParameter("Invalid border size!");
				break;
			case BP_UNIDARK:
			case BP_UNILIGHT:
			case BP_TICK_RAISED:
			case BP_TICK_SUNKED:
				if (size >= 1)
					m_iSize = size;
				else 
					throw new InvalidParameter("Invalid border size!");
				break;
		}
		
		switch (m_iStyle) {
			case BP_RAISED:
			case BP_SUNKED:
				m_iBorder = 2 + m_iSize;
				break;
			case BP_FRAME:
				m_iBorder = 2 + m_iSize + 2;
				break;
			case BP_UNIDARK:
			case BP_UNILIGHT:
			case BP_TICK_RAISED:
			case BP_TICK_SUNKED:
				m_iBorder = m_iSize;
				break;
		}
		repaint();
	}
	
	/**
	 * Gets the dark color used to draw this panel border.
	 * 
	 * @return The dark border color.
	 * @see #setDarkBorder(java.awt.Color)
	 */
	public Color getDarkBorder()
	{
		return m_ColorDark;
	}
	
	/**
	 * Sets the dark color used to draw this panel border.
	 * 
	 * @param color Dark border color to use.
	 * @see #getDarkBorder()
	 */
	public void setDarkBorder(Color color)
	{
		m_ColorDark = color;
		repaint();
	}
	
	/**
	 * Gets the very dark color used to draw this panel border.
	 * 
	 * @return The very dark border color.
	 * @see #setDarkDkBorder(java.awt.Color)
	 */
	public Color getDarkDkBorder()
	{
		return m_ColorDarkDk;
	}
	
	/**
	 * Sets the very dark color used to draw this panel border.
	 * 
	 * @param color Very dark border color to use.
	 * @see #getDarkDkBorder()
	 */
	public void setDarkDkBorder(Color color)
	{
		m_ColorDarkDk = color;
		repaint();
	}
	
	/**
	 * Gets the light color used to draw this panel border.
	 * 
	 * @return The light border color.
	 * @see #setLightBorder(java.awt.Color)
	 */
	public Color getLightBorder()
	{
		return m_ColorLight;
	}
	
	/**
	 * Sets the light color used to draw this panel border.
	 * 
	 * @param color Light border color to use.
	 * @see #getLightBorder()
	 */
	public void setLightBorder(Color color)
	{
		m_ColorLight = color;
		repaint();
	}
	
	/**
	 * Gets the very light color used to draw this panel border.
	 * 
	 * @return The very light border color.
	 * @see #setLightLtBorder(java.awt.Color)
	 */
	public Color getLightLtBorder()
	{
		return m_ColorLightLt;
	}
	
	/**
	 * Sets the very light color used to draw this panel border.
	 * 
	 * @param color Very light border color to use.
	 * @see #getLightltBorder()
	 */
	public void setLightLtBorder(Color color)
	{
		m_ColorLightLt = color;
		repaint();
	}
}
