//******************************************************************************
// Project: Simple LAN Monitor v1.00
// URL:		http://slam.sourceforge.net
// File:    GraphPanel.java
//
// Description:
//          Creates a panel to display a lines graphic with the values returned 
//          by the server software
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.*;
import java.lang.Double.*;
import java.lang.Math.*;

/**
 * Creates a panel to display a lines graphic with the values returned 
 * by the server software
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.0, 2001-10-21
 * @since	1.0
 */
public class GraphPanel extends Component
{
	private final int MAX_NUM = 10;
	private final int MAX_LIN = 6;
	private final int BORDER = 4;
	private final int GAP = 10;
	
	private double[] m_dData = new double[MAX_NUM];
	private int m_iIndex;
	private int m_iDeltaTime;
	private String m_strTime = "";
	
	private static long m_lCount = 0;	// Instances counter
	private Image m_imageOffScreen = null;	// Window 'Off Screen' image buffer
	
	public GraphPanel()
	{
		super();

		this.setName("graphpanel" + String.valueOf(m_lCount));
		++m_lCount;
		
		m_iIndex = 0;
		m_iDeltaTime = 0;
		
		for (int i = 0; i < MAX_NUM; i++) {
			m_dData[i] = 0.0;
		}
		
		this.setBackground((Color)SystemColor.white);
		this.setForeground((Color)SystemColor.black);
		this.setFont(new java.awt.Font("Courier", Font.BOLD, 10));
	}
	
	public void setTime(int t, String s)
	{
		m_iDeltaTime = t;
		m_strTime = s;
	}
	
	public synchronized void resetIndex()
	{
		m_iIndex = 0;
	}
	
	public synchronized void addValue(double v)
	{
		if (m_iIndex < MAX_NUM) {
			m_dData[m_iIndex] = v;
			++m_iIndex;
		}
		else {
			for (int i = 1; i < MAX_NUM; i++) {
				m_dData[i - 1] = m_dData[i];
			}
			m_dData[MAX_NUM - 1] = v;
		}
			
		this.repaint();
	}
	
	/**
	 * Invalidates the container.
	 * <p>
	 * This implementation destroys our bitmap image 
	 * buffer of this panel's window client area.
	 * 
	 * @see java.awt.Container#invalidate()
	 */
	public void invalidate() {
		super.invalidate();
		m_imageOffScreen = null;
	}
	
	/**
	 * Updates the container.
	 * <p>
     * This implementation avoids the background to 
     * be erased by calling <code>paint</code> insted.
	 * 
	 * @param	Graphics object.
	 * @see		java.awt.Container#update(java.awt.Graphics)
	 * @see		#paint(java.awt.Graphics)
	 */
	public void update(Graphics g) {
		paint(g);
	}
	
	/**
	 * Paints the container.
	 * <p>
	 * This implementation paints this panel window 
	 * and all its children at once, using an m_imageOffScreen bitmap image buffer.
	 * <p>
	 * Doing so, avoids flicking of this window and its children.
	 * 
	 * @param	Graphics object.
	 * @see		java.awt.Container#paint(java.awt.Graphics)
	 */
	public synchronized void paint(Graphics g)
	{
		final String kb = " Kb/s";
		int xText;
		int yText;
		int x0;
		int y0;
		int x1;
		int y1;
		int lenX;
		int lenY;
		int incrX;
		int incrY;
		int xC;
		int yC;
		int gap;
		int yMax;	
		double aux;
			
		// Variable inicialization
		Rectangle rect = this.getBounds();
		rect.x = 0;
		rect.y = 0;
		gap = GAP / 2;
		aux = m_dData[0];
		for (int i = 1; i < m_iIndex; i++) {
			if (m_dData[i] > aux)
				aux = m_dData[i];
		}
		yMax = (int)java.lang.Math.ceil(aux);
		
		if (yMax <= 0)
			yMax = 1;
		
		// Creates an 'Off Screen' buffer
		if (m_imageOffScreen == null) {
			m_imageOffScreen = createImage(rect.width, rect.height);
		}
		Graphics og = m_imageOffScreen.getGraphics();
		og.setClip(0, 0, rect.width, rect.height);
		
		// Clear background
		og.setColor(getBackground());
		og.fillRect(0, 0, rect.width, rect.height);
				
		// Paint children
		super.paint(og);
						
		/* Paint the stuf */
		// Corrects size
		rect.width = rect.width - 1;
		rect.height = rect.height - 1;
		
		// Font stuf
		FontMetrics fm = og.getFontMetrics();
		xText = fm.stringWidth(Double.toString(yMax) + kb + "  ");
		yText = fm.getHeight();
		x0 = BORDER + xText;
		y0 = BORDER + gap;
		x1 = rect.width - BORDER - gap;
		y1 = rect.height - 2 * yText - BORDER;
		lenX = x1 - x0;
		lenY = y1 - y0;
		incrX = lenX / (MAX_NUM - 1);
		incrY = lenY / (MAX_LIN - 1);
        xC = x0;
		og.setColor((Color)SystemColor.black);
		// XX's axe
		og.drawLine(x0 - gap, y1, x1 + gap, y1);
		og.drawLine(x1 + gap, y1, x1 + gap - 3, y1 - 3);
		og.drawLine(x1 + gap, y1, x1 + gap - 3, y1 + 3);
		// YY's axe
		og.drawLine(xC, y0 - gap, xC, y1 + gap);
		og.drawLine(xC, y0 - gap, xC - 3, y0 - gap + 3);
		og.drawLine(xC, y0 - gap, xC + 3, y0 - gap + 3);
		// YY's scale
		aux = (double)yMax / (double)(MAX_LIN - 1);
		og.drawString("0.0" + kb, xC - xText, y1 + (yText / 4));
		for (int i = y1 - incrY; i > y0; i = i - incrY) {
			og.drawLine(xC - gap, i, xC + gap, i);
			og.drawString(Double.toString(java.lang.Math.ceil(aux * 100.0) / 100.0)  + " Kb/s", xC - xText, i + (yText / 4));
			aux = aux + (double)yMax / (double)(MAX_LIN - 1);
		}
		// XX's scale
		for (int i = x0; i < x1; i = i + incrX) {
			og.drawLine(i, y1 - gap, i, y1 + gap);
		}
		String str = Integer.toString(m_iDeltaTime) + m_strTime;
		xText = fm.stringWidth(str);
		og.drawString(str, x1 - xText - BORDER, y1 + yText + gap);
		// The lines
		og.setColor((Color)SystemColor.blue);
		for (int i = 0 ; i < m_iIndex - 1; i++) {
			og.drawLine(xC + i * incrX, y1 - (int)((m_dData[i] / (double)yMax) * (double)lenY), xC + (i + 1) * incrX, y1 - (int)((m_dData[i + 1] / (double)yMax) * (double)lenY));
		}
		
		// Blast entire image to screen
		g.drawImage(m_imageOffScreen, 0, 0, null);
		og.dispose();
	}
}
