//******************************************************************************
// Project: Simple LAN Monitor v1.10
// URL:		http://slam.sourceforge.net
// File:    MessageBox.java
//
// Description:
//          Creates and manages an About Dialog Box, with the authors and the
//          program information.
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.MediaTracker;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Toolkit;
import java.awt.FlowLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Component;
import java.awt.Panel;
import java.awt.Image;
import java.awt.Button;
import java.awt.TextArea;
import java.awt.SystemColor;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.ResourceBundle;

import Icon;

/**
 * Creates and manages an <i>Message Box Dialog</i>, with the text suplied.
 * 
 * This class is derived from <b>java.awt.Dialog<b>.
 * 
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.10, 2001-10-21
 * @since	1.10
 */
public class MessageBox extends Dialog
					    implements WindowListener,
	                               ActionListener,
	                               FocusListener {
	// Public constants
	
	/**
	 * <b>Yes</b> button flag.
	 */
	public final static int MB_YES		= 0x00000001;
	/**
	 * <b>No</b> button flag.
	 */
	public final static int MB_NO		= 0x00000002;
	/**
	 * <b>OK</b> button flag.
	 */
	public final static int MB_OK		= 0x00000004;
	/**
	 * <b>Abort</b> button flag.
	 */
	public final static int MB_ABORT	= 0x00000008;
	/**
	 * <b>Retry</b> button flag.
	 */
	public final static int MB_RETRY	= 0x00000010;
	/**
	 * <b>Ignore</b> button flag.
	 */
	public final static int MB_IGNORE	= 0x00000020;
	/**
	 * <b>Cancel</b> button flag.
	 */
	public final static int MB_CANCEL	= 0x00000040;
	/**
	 * <b>Yes</b> and <b>No</b> buttons flags combined.
	 */
	public final static int MB_YES_NO			= MB_YES | MB_NO;
	/**
	 * <b>Yes</b>, <b>No</b> and <b>Cancel</b> buttons flags combined.
	 */
	public final static int MB_YES_NO_CANCEL	= MB_YES | MB_NO | MB_CANCEL;
	/**
	 * <b>OK</b> and <b>Cancel</b> buttons flags combined.
	 */
	public final static int MB_OK_CANCEL		= MB_OK | MB_CANCEL;
	/**
	 * <b>Retry</b> and <b>Cancel</b> buttons flags combined.
	 */
	public final static int MB_RETRY_CANCEL		= MB_RETRY | MB_CANCEL;
	/**
	 * <b>Abort</b>, <b>Retry</b> and <b>Ignore</b> buttons flags combined.
	 */
	public final static int MB_ABORT_RETRY_IGNORE= MB_ABORT | MB_RETRY | MB_IGNORE;
	
	/**
	 * Value returned by <code>getButtonPressed()</code>, indicating
	 * that the Message Box dialog was closed by pressing the
	 * close/quit window button in the dialog (the <b>x</b> in 
	 * Windows 9x/NT 4 and some X Windows window managers).
	 * 
	 * @see #getButtonPressed()
	 */
	public final static int MB_WINDOW_CLOSED	= 0xFFFFFFFF;
	
	/**
	 * Exclamation icon flag.
	 * 
	 * This flag shows an <img src="images/CaetanoSoft/Exclamation.gif">
	 * in the Message Box.
	 */
	public final static int MB_ICON_EXCLAMATION	= 0x00010000;
	/**
	 * Question icon flag.
	 * 
	 * This flag shows an <img src="images/CaetanoSoft/Question.gif">
	 * in the Message Box.
	 */
	public final static int MB_ICON_QUESTION	= 0x00020000;
	/**
	 * Information icon flag.
	 * 
	 * This flag shows an <img src="images/CaetanoSoft/Information.gif">
	 * in the Message Box.
	 */
	public final static int MB_ICON_INFORMATION	= 0x00040000;
	/**
	 * Stop icon flag.
	 * 
	 * This flag shows an <img src="images/CaetanoSoft/Stop.gif">
	 * in the Message Box.
	 */
	public final static int MB_ICON_STOP		= 0x00080000;
	
	/**
	 * Plays a beep wend the Message Box is displayed.
	 */
	public final static int MB_SOUND_BEEP		= 0x80000000;
	
	/**
	 * Format flags.
	 */
	public final static int MB_FLAGS_BUTTONS	= 0x0000FFFF;
	public final static int MB_FLAGS_ICONS		= 0x00FF0000;
	public final static int MB_FLAGS_SOUNDS		= 0xFF000000;
	
	// Private members
	// Resources
	private ResourceBundle m_ResourcesBundle = ResourceBundle.getBundle("MBResources", getLocale());
	
	// Buttons, TextArea and Icon
	private Button m_btnYes		= new Button(m_ResourcesBundle.getString("YesKey"));
	private Button m_btnNo		= new Button(m_ResourcesBundle.getString("NoKey"));
	private Button m_btnOK		= new Button(m_ResourcesBundle.getString("OKKey"));
	private Button m_btnAbort	= new Button(m_ResourcesBundle.getString("AbortKey"));
	private Button m_btnRetry	= new Button(m_ResourcesBundle.getString("RetryKey"));
	private Button m_btnIgnore	= new Button(m_ResourcesBundle.getString("IgnoreKey"));
	private Button m_btnCancel	= new Button(m_ResourcesBundle.getString("CancelKey"));
	private TextArea m_txtaMessage = new TextArea();
	private Icon m_Icon	    = null;
	
	// Private stuf
	private static long m_lCount = 0;	// Instances counter
	private static boolean m_bRunning = false;
	
	private int m_iStyle = 0;			// Window style
	
	/**
	 * Button chosen.
	 */
	private int m_iRet = 0;
	
	
	// Constructors
	
	/**
	 * Constructs a Message Box Dialog and show it. This dialog can
	 * have an optional icon (<i>Exclamation</i>, <i>Question</i>, 
	 * <i>Information</i> or <i>Stop</i>), any combination of buttons 
	 * (by this orther of aperance: <i>Yes</i>, <i>No</i>, <i>OK</i>, 
	 * <i>Abort</i>, <i>Retry</i>, <i>Ignore</i> and <i>Cancel</i>) and 
	 * play and optional beep sound wend displayed.
	 * <p>
	 * If parent frame is <code>null</code>, the desktop window is 
	 * considered insted.
	 * <p>
	 * The dialog style flags is a combination of the <code>MB_</code>*
	 * flags or'ed together in this maner:
	 * <pre>
	 *     - If a beep sound is required wend the dialog is show:
	 *         <code>MB_SOUND_BEEP</code>
	 * 
	 *     - One of this flags, if the respective icon is desirable:
	 *         <code>MB_ICON_EXCLAMATION</code>
	 *         <code>MB_ICON_QUESTION</code>
	 *         <code>MB_ICON_INFORMATION</code>
	 *         <code>MB_ICON_STOP</code>
	 * 
	 *     - Any combination of this flags, for the respective(s) button(s) 
	 *       to be displayed:
	 *         <code>MB_YES</code>
	 *         <code>MB_NO</code>
	 *         <code>MB_OK</code>
	 *         <code>MB_RETRY</code>
	 *         <code>MB_IGNORE</code>
	 *         <code>MB_CANCEL</code>
	 *         <code>MB_YES_NO</code>
	 *         <code>MB_YES_NO_CANCEL</code>
	 *         <code>MB_OK_CANCEL</code>
	 *         <code>MB_RETRY_CANCEL</code>
	 *         <code>MB_ABORT_RETRY_IGNORE</code>
	 * 
	 * <b>Examples:</b>
	 *     - An dialog with a beep, a Question icon and the Yes, No buttons:
	 *         <code>MB_SOUND_BEEP | MB_ICON_QUESTION | MB_YES | MB_NO</code>
	 *     - An dialog with a Information icon and the Yes, No, Cancel, Retry 
	 *       buttons:
	 *         <code>MB_ICON_QUESTION | MB_YES_NO_CANCEL | MB_RETRY</code>
	 *     - An dialog with a beep sound, no icon and the Yes, No buttons:
	 *         <code>MB_SOUND_BEEP | MB_YES_NO</code>
	 * 
	 * <b>Note:</b>
	 *     If no button flag is given, <code>MB_OK</code> is assumed by default.
	 * </pre>
	 * 
	 * @param frame	Parent frame. Can't be null.
	 * @param title	Title on the Dialog Window.
	 * @param text Message to display.
	 * @param flags	Dialog style flags.
	 */
	public MessageBox(Frame frame, String title, String text, int flags)
	{
		super(frame, title, true);

		// Only one instance can run at a time
		if (m_bRunning) {
			m_iRet = 0;
			this.dispose();
			return;
		}
		else {
			m_bRunning = true;
				
			// Give a name to this message box
			this.setName("messagebox" + String.valueOf(m_lCount));
			++m_lCount;
		
			m_txtaMessage.setText(text);
			m_iStyle = flags;
			
			// Disable parent window, while constructing this dialog
			boolean b = frame.isEnabled(); // Parent enabled state.
			frame.setEnabled(false);

			DialogInit();
			
			// Resets the enableld state of the parent window
			frame.setEnabled(b);
			this.setVisible(true);
		}
	}
		
	// MessageBox methods
	
	/**
	 * Returns the code of the pressed button or
	 * MB_WINDOW_CLOSED, if the window has closed. 
	 * 
	 * @return MB_WINDOW_CLOSED, MB_YES, MB_NO, MB_OK, MB_ABORT, MB_IGNORE, MB_RETRY, MB_CANCEL
	 */
	public int getButtonPressed()
	{
		return m_iRet;	
	}
	
	public synchronized boolean isRunning()
	{
		return m_bRunning;	
	}
	
	private void DialogInit()
	{		
		String text = m_txtaMessage.getText();	
			
		// Atributs set up
		if ((m_iStyle & MB_FLAGS_BUTTONS) == 0)
			m_iStyle = m_iStyle | MB_OK;
		
		// Colors
		this.setBackground((Color)SystemColor.control);
		this.setForeground((Color)SystemColor.controlText);
		
		// TextArea setup
		m_txtaMessage.addFocusListener(this);
		m_txtaMessage.setEditable(false);

		// Add buttons
		Panel p = new Panel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER, 4, 2));
		if ((m_iStyle & MB_YES) != 0) {
			m_btnYes.addActionListener(this);
			m_btnYes.setSize(50, 30);
			p.add(m_btnYes);
		}
		if ((m_iStyle & MB_NO) != 0) {
			m_btnNo.addActionListener(this);
			m_btnNo.setSize(50, 30);
			p.add(m_btnNo);
		}
		if ((m_iStyle & MB_OK) != 0) {
			m_btnOK.addActionListener(this);
			m_btnOK.setSize(50, 30);
			p.add(m_btnOK);
		}
		if ((m_iStyle & MB_ABORT) != 0) {
			m_btnAbort.addActionListener(this);
			m_btnAbort.setSize(50, 30);
			p.add(m_btnAbort);
		}
		if ((m_iStyle & MB_RETRY) != 0) {
			m_btnRetry.addActionListener(this);
			m_btnRetry.setSize(50, 30);
			p.add(m_btnRetry);
		}
		if ((m_iStyle & MB_IGNORE) != 0) {
			m_btnIgnore.addActionListener(this);
			m_btnIgnore.setSize(50, 30);
			p.add(m_btnIgnore);
		}
		if ((m_iStyle & MB_CANCEL) != 0) {
			m_btnCancel.addActionListener(this);
			m_btnCancel.setSize(50, 30);
			p.add(m_btnCancel);
		}
		
		// Image (icon) path
		String strImage = "/MBResources/";
		Image image = null;
		switch (m_iStyle & MB_FLAGS_ICONS) {
			case 0:
				strImage = null;
				break;
			case MB_ICON_EXCLAMATION:
				strImage = strImage + "Exclamation.gif";
				break;
			case MB_ICON_INFORMATION:
				strImage = strImage + "Information.gif";
				break;
			case MB_ICON_QUESTION:
				strImage = strImage + "Question.gif";
				break;
			case MB_ICON_STOP:
				strImage = strImage + "Stop.gif";
				break;
			default:
				strImage = null;
				System.err.println("MessageBox->DialogInit: Bad Icon type.");
				System.err.println("    " + (m_iStyle & MB_FLAGS_ICONS));
		}
		
		// Load the image
		if (strImage != null) {
			URL appURL = getClass().getResource(strImage);
			image = Toolkit.getDefaultToolkit().getImage(appURL);
			MediaTracker tracker = new MediaTracker(this);
			tracker.addImage(image, 0);
			try {
				tracker.waitForAll();
				if (tracker.isErrorAny()) {
					// Error loading image
					image = null;
					System.err.println("MessageBox->DialogInit: Image not found.");
					System.err.println("    " + strImage);
				}
			}
			catch (InterruptedException e) {
				// Application cancel
				System.err.println(e.getMessage());
				e.printStackTrace();
				image = null;
			}
		}
		
		// Calculates the area of the text
		String strLF = System.getProperty("line.separator");
		int c = 0;
		int l = 0;
		int r1 = 0;
		int r2 = 0;
		int i;
		int j;
		String str;
		if (text != null) {
			do {
				r1 = text.indexOf(strLF, r1);
				if (r1 == -1)
					r1 = text.length();
				++l;
				i = r1 - r2;
				try {
					// If the text contains tabs ('\t'), increments i
					// Tabs is consider to be 8 spaces
					str = text.substring(r2, r1);
					j = 0;
					do {
						j = str.indexOf((int)'\t', j);
						if (j != -1) {
							i += 7;
							++j;
						}
					} while (j != -1);
				}
				catch (StringIndexOutOfBoundsException e) {
					System.err.println(e.getMessage());
					e.printStackTrace();
				}
				if (c < i)
					c = i;
				r1 = r1 + strLF.length();
				r2 = r1;
			} while (r1 < text.length());
		}
		if (l < 3)
			l = 3;
		if (c < 10)
			c = 10;
		m_txtaMessage.setColumns(c);
		m_txtaMessage.setRows(l);
				
		// Add components to frame window.
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		this.setLayout(gb);

		// Icon
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.fill = GridBagConstraints.NONE;
		gbc.gridx = GridBagConstraints.RELATIVE;
		gbc.gridy = GridBagConstraints.RELATIVE;
		gbc.gridwidth = 1;
		gbc.gridheight = 3;
		gbc.ipadx = 0;
		gbc.ipady = 0;
		gbc.insets = new Insets(4, 4, 4, 4);
		gbc.weightx = 0;
		gbc.weighty = 0;
		
		if (image != null) {
			m_Icon = new Icon(image, 32, 32);
			gbc.insets.right = 0;
			gb.setConstraints(m_Icon, gbc);
			this.add(m_Icon);
			gbc.insets.right = 4;
		}
		
		// Mesage text
		gbc.fill = GridBagConstraints.BOTH;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		
		gb.setConstraints(m_txtaMessage, gbc);
		add(m_txtaMessage);

		// Buttons panel
		gbc.gridheight = 1;		
		
		gb.setConstraints(p, gbc);
		this.add(p);

		// Show this dialog
		this.addWindowListener(this);
		this.pack();
		this.setResizable(false);
		if ((m_iStyle & MB_SOUND_BEEP) != 0) {
			Toolkit.getDefaultToolkit().beep();
		}
	}
	
	// Component methods
	
	public String paramString()
	{
		return new String(this.getName() + ", title=" + this.getTitle() + ", style=" + String.valueOf(m_iStyle));
	}

	// ActionListener methods
	
	public void actionPerformed(ActionEvent e)
	{
		Object objSource = e.getSource();
		// Buttons
		if (objSource instanceof Button) {
			Button buttonS = (Button)objSource;
			if (buttonS.equals(m_btnYes)) {
				m_iRet = MB_YES;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnNo)) {
				m_iRet = MB_NO;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnOK)) {
				m_iRet = MB_OK;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnAbort)) {
				m_iRet = MB_ABORT;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnRetry)) {
				m_iRet = MB_RETRY;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnIgnore)) {
				m_iRet = MB_IGNORE;
				m_bRunning = false;
				dispose();
				return;
			}
			if (buttonS.equals(m_btnCancel)) {
				m_iRet = MB_CANCEL;
				m_bRunning = false;
				dispose();
				return;
			}
		}
	}
	
	// WindowListener methods
	
	public void windowOpened(WindowEvent e)
	{
	}

	public void windowClosed(WindowEvent e)
	{
	}
	
	public void windowClosing(WindowEvent e)
	{
		if (e.getWindow().equals((Window)this)) {
			m_iRet = MB_WINDOW_CLOSED;
			m_bRunning = false;
			this.dispose();
		}
	}
	
	public void windowActivated(WindowEvent e)
	{
	}
	
	public void windowDeactivated(WindowEvent e)
	{
	}
	
	public void windowIconified(WindowEvent e)
	{
	}
	
	public void windowDeiconified(WindowEvent e)
	{
	}
	
	// FocusListener methods
	
	public void focusGained(FocusEvent e)
	{
		if (e.getComponent().equals(m_txtaMessage)) {
			m_txtaMessage.transferFocus();
		}	
	}
	
	public void focusLost(FocusEvent e)
	{
	}
}
