//******************************************************************************
// Project: Simple LAN Monitor v1.00
// URL:		http://slam.sourceforge.net
// File:    SimpleLANMonitor.java
//
// Description:
//          Creates an applet to display the results returned by the 'Simple LAN
//          Monitor' server.
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.security.*;
import java.util.ResourceBundle;

import BorderPanel.*;
import GraphPanel.*;
import AboutBox.*; 

/**
 * Creates an applet to display the results returned by the 'Simple LAN Monitor' server.
 * 
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.0, 2001-10-21
 * @since	1.0
 */
public class SimpleLANMonitorApplet extends Applet implements Runnable, ActionListener
{
	// Default values for program parameters
	private final String m_strDefaultIP = "127.0.0.1";
	private final int m_iDefaultiDeltaTime = 60;
	private final int m_iDefaultPort = 5000;

	// HTML parameters names
	private	final String m_strTitleParam = "title";
	private	final String m_strDeltaTimeParam = "deltatime";
	private	final String m_strIPParam = "ip";
	private	final String m_strPortParam = "port";
	
	// Program parameters m_dData
	private String m_strTitle;
	private int m_iDeltaTime;
	private int m_iPort;
	private String m_strIP;
	
	// GUI
	private Label m_labelTitle = new Label();
	private Label m_labelStatus = new Label();
	private Button m_buttonStop = new Button();
	private Button m_buttonAbout = new Button();
	private GraphPanel m_graphPanel = new GraphPanel();
	
	// Internacional resourses support
	private ResourceBundle m_ResourcesBundle = null;
	
	// Thread support
	private Thread m_threadSimpleLANMonitor = null;
	private boolean m_bStop = false; 

	/**
	 * The entry point for the applet. 
	 */
	public void init()
	{
		// Resources
		m_ResourcesBundle = ResourceBundle.getBundle("StringResources", getLocale());
		// Load parameters from HTML page
		getPageParams();
		// Show the Applet
		initForm();
		// Create working thread
		if (m_threadSimpleLANMonitor == null) {
			m_threadSimpleLANMonitor = new Thread(this);
			m_threadSimpleLANMonitor.start();
		}
	}
	
	/**
	 * Place additional applet clean up code here.  destroy() is called when
	 * when you applet is terminating and being unloaded.
	 */
	public void destroy()
	{
		if (m_threadSimpleLANMonitor != null) {
			m_threadSimpleLANMonitor = null;
		}
	}

	/**
	 * Returns a string describing the applet's author,
	 * copyright date, or miscellaneous information.
     */
	public String getAppletInfo()
	{
		return "Name: Simple LAN Monitor Applet\r\n\r\n" +
			   "Version: 1.00\r\n\r\n" +
		       "Authors:\r\n" + 
			   "    Jos Caetano A. C. Silva\r\n" +
			   "    Rui Fernando Ribeiro\r\n\r\n" +      
			   "E-mail:\r\n"+
			   "    jcaetano@users.sourceforge.net\r\n" +
			   "    ruiribeiro@users.sourceforge.net\r\n";
	}
	
	/**
	 * Reads parameters from the applet's HTML host and sets applet
	 * properties.
	 */
	private void getPageParams()
	{
		String str = new String();
		
		// Title
		m_strTitle = getParameter(m_strTitleParam);
		if (m_strTitle == null) {
			m_strTitle = m_ResourcesBundle.getString("DefaultTitleKey");
		}
		
		// Delta time
		str = getParameter(m_strDeltaTimeParam);
		if ((str == null) || (str == "")) {
			m_iDeltaTime = m_iDefaultiDeltaTime;
		}
		else {
			try {
				m_iDeltaTime = Integer.parseInt(str);
			}
			catch(NumberFormatException e) {
				m_iDeltaTime = m_iDefaultiDeltaTime;
			}
		}
		
		// TCP/IP port
		str = getParameter(m_strPortParam);
		if ((str == null) || (str == "")) {
			m_iPort = m_iDefaultPort;
		}
		else {
			try {
				m_iPort = Integer.parseInt(str);
			}
			catch(NumberFormatException e) {
				m_iPort = m_iDefaultPort;
			}
		}
		
		// TCP/IP server
		m_strIP = getParameter(m_strIPParam);
		if ((m_strIP == null) || (m_strIP == "")) {
			m_strIP = m_strDefaultIP;
		}
	}

	/**
	 * External interface used by design tools to show properties of an applet.
	 */
	public String[][] getParameterInfo()
	{
		String[][] info =
		{
			{ m_strTitleParam, "String", "Title string to be displayed" },
			{ m_strDeltaTimeParam, "int", "Delta time (in seconds) to request new data from server"},
			{ m_strIPParam, "String", "IP of the host server"},
			{ m_strPortParam, "int", "TCP/IP port to connect"},
		};
		return info;
	}

	/**
	 * Intializes values for the applet and its components
	 */
	void initForm()
	{
		Font f = new Font("Courrier", Font.BOLD, 16);
		
		m_labelTitle.setFont(f);
		m_labelTitle.setAlignment(Label.CENTER);
		m_labelTitle.setText(m_strTitle);
		
		m_labelStatus.setSize(new Dimension(this.getSize().width - 30, 20));
		m_labelStatus.setText(m_ResourcesBundle.getString("LoadingKey"));
					
		m_buttonStop.addActionListener(this);
		m_buttonStop.setLabel(m_ResourcesBundle.getString("StopKey"));
		
		m_buttonAbout.addActionListener(this);
		m_buttonAbout.setLabel(m_ResourcesBundle.getString("AboutKey"));
		
		Panel p = new Panel();
		p.setLayout(new FlowLayout());
		p.add(m_buttonStop);
		p.add(m_buttonAbout);
		
		BorderPanel p1 = null;
		BorderPanel p2 = null;
		
		m_graphPanel.setTime(m_iDeltaTime, m_ResourcesBundle.getString("TimeKey"));
		
		try {
			p2 = new BorderPanel(BorderPanel.BP_FRAME, 4);
			p2.setLayout(new BorderLayout());
			p1 = new BorderPanel(BorderPanel.BP_SUNKED, 4);
			p1.setLayout(new BorderLayout());
		}
		catch (InvalidParameter e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		
		p2.add("Center", m_graphPanel);
		p1.add("Center", m_labelStatus);
		p1.add("East", p);

		this.setBackground((Color)SystemColor.control);
		this.setForeground((Color)SystemColor.controlText);
		this.setLayout(new BorderLayout());
		this.add("North", m_labelTitle);
		this.add("Center", p2);
		this.add("South", p1);
	}
	
	/**
	 * The start() method is called when the page containing the applet
	 * first appears on the screen. The AppletWizard's initial implementation
	 * of this method starts execution of the applet's thread.
	 */
	public void start()
	{
		m_labelStatus.setText(m_ResourcesBundle.getString("AppStartingKey"));
				
		if (m_threadSimpleLANMonitor == null) {
			m_threadSimpleLANMonitor = new Thread(this);
			m_threadSimpleLANMonitor.start();
		}
		m_labelStatus.setText(m_ResourcesBundle.getString("AppStartedKey"));
	}
	
	/**
	 * The stop() method is called when the page containing the applet is
	 * no longer on the screen. The AppletWizard's initial implementation of
	 * this method stops execution of the applet's thread.
	 */
	public void stop()
	{
		m_labelStatus.setText(m_ResourcesBundle.getString("AppStopingKey"));
			
		if (m_threadSimpleLANMonitor != null) {
			m_threadSimpleLANMonitor = null;
		}
		m_labelStatus.setText(m_ResourcesBundle.getString("AppStopedKey"));
	}

	/**
	 * The run() method is called when the applet's thread is started. If
	 * your applet performs any ongoing activities without waiting for user
	 * input, the code for implementing that behavior typically goes here. For
	 * example, for an applet that performs animation, the run() method controls
	 * the display of images.
	 */
	public void run()
	{
		int i;
		byte[] byteStream = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
		Double d;
		Socket socket = null;
		InputStream inputStream = null;
		
		Thread thisThread = Thread.currentThread();
		while(!m_bStop) {
			try {
				socket = new Socket(m_strIP, m_iPort);
				inputStream = socket.getInputStream();
				m_labelStatus.setText(m_ResourcesBundle.getString("ConnectedKey") + m_strIP + m_ResourcesBundle.getString("PortKey") + m_iPort);

				i = inputStream.read(byteStream);
				if (i > 0) {
					String s = new String(byteStream, 0, i);
					try {
						d = new Double(s);
						if (!d.isNaN()) {
							m_graphPanel.addValue(d.doubleValue());
						}
					}
					catch (NumberFormatException e) {
						// OK
					}
					s = null;
				}
			}
			catch(IOException e) {
				m_labelStatus.setText(e.getMessage());
				System.err.println(e.getMessage());
				System.err.println(e.toString());
				e.printStackTrace();
			}
			catch(SecurityException e) {
				m_labelStatus.setText(e.getMessage());
				System.err.println(e.getMessage());
				System.err.println(e.toString());
				e.printStackTrace();
			}
			finally {
				try {
					if (inputStream != null) {
						inputStream.close();
						inputStream = null;
					}
				}
				catch(IOException e) {
					m_labelStatus.setText(e.getMessage());
					System.err.println(e.getMessage());
					System.err.println(e.toString());
					e.printStackTrace();
				}
				try {
					if (socket != null) {
						socket.close();
						socket = null;
					}
				}
				catch(IOException e) {
					m_labelStatus.setText(e.getMessage());
					System.err.println(e.getMessage());
					System.err.println(e.toString());
					e.printStackTrace();
				}
			}
			// Sleep
			try {
				Thread.sleep(m_iDeltaTime * 1000);
			}
			catch (InterruptedException e) {
				// OK, no error
			}
		}
	}
	
	// ActionListener methods
	public void actionPerformed(ActionEvent e)
	{
		Object objSource = e.getSource();
		// Menu Items
		if (objSource instanceof Button) {
			Button b = (Button)objSource;
			if (b.equals(m_buttonStop)) {
				if (m_bStop) {
					m_buttonStop.setLabel(m_ResourcesBundle.getString("StopKey"));
					m_bStop = false;
					this.start();
				}
				else {
					m_buttonStop.setLabel(m_ResourcesBundle.getString("StartKey"));
					m_bStop = true;
					this.stop();
					m_graphPanel.resetIndex();
				}
			}
			if (b.equals(m_buttonAbout)) {
				AboutBox d = new AboutBox(m_buttonAbout.getLabel());
			}
		}
	}
}
