//******************************************************************************
// Project: Simple LAN Monitor v1.10
// URL:		http://slam.sourceforge.net
// File:    Icon.java
//
// Description:
//          Creates and manages an About Dialog Box, with the authors and the
//          program information.
//
// Copyright (C) 2001:
//          Jos Caetano Silva (jcaetano@users.sourceforge.net)
//          Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
//
// Licence:
//          This program is free software; you can redistribute it and/or modify
//          it under the terms of the GNU General Public License as published by
//          the Free Software Foundation (read the 'GNU General Public 
//          License.htm' file, for more details).
// 
//          This program is distributed in the hope that it will be useful,
//          but WITHOUT ANY WARRANTY; without even the implied warranty of
//          MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//          GNU General Public License for more details.
// 
//          You should have received a copy of the GNU General Public License
//          along with this program; if not, write to the Free Software
//          Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//******************************************************************************

import java.awt.Image;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Component;

/**
 * Creates and manages an Icon (java.awt.Image) component
 * or an empty transparent space area, that can be used 
 * as a separator.
 * 
 * @author	Jos Caetano Silva (jcaetano@users.sourceforge.net)
 * @author	Rui Fernando Ribeiro (ruiribeiro@users.sourceforge.net)
 * @version 1.10, 2001-10-21
 * @since	1.10
 */
public class Icon extends Component {
	// Public constants
	
	/**
	 * Defaul width and height size: 32.
	 */
	public final static int DEFAULT_SIZE = 32;
	
	// Private members
	
	private static long m_lCount = 0;	// Instances counter
	
	private Image m_Image = null;		// Icon image
	private int m_x = 0;				// Icon width
	private int m_y = 0;				// Icon height
	
	// Constructors
	
	/**
	 * Constructs an Icon Component with the default
	 * dimentions (DEFAULT_SIZE x DEFAULT_SIZE) and 
	 * the image <i>img</i>.
	 * 
	 * @param img The image to display on it.
	 * 
	 * @see #Icon(java.awt.Image, int, int)
	 * @see #DEFAULT_SIZE
	 */
	public Icon(Image img)
	{
		this(img, DEFAULT_SIZE, DEFAULT_SIZE);
	}
	
	/**
	 * Constructs an Icon Component with the dimentions
	 * <i>x</i> by <i>y</i> and the image <i>img</i>.
	 * 
	 * If <i>img</i> is <i>null</i>, an empty tranparent
	 *  area is used.
	 * 
	 * @param img	The image to display on it.
	 * @param x		The	width of the icon.
	 * @param y		The	height of the icon.
	 */
	public Icon(Image img, int x, int y)
	{
		super();
		
		m_Image = img;
		m_x = x;
		m_y = y;
		
		// Give a name to this message box
		this.setName("icon" + String.valueOf(m_lCount));
		++m_lCount;
		
		// Size
		this.setSize(m_x, m_y);
	}
	
	// Component methods
	
	/**
	 * If an image is given, draws it, otherwise do
	 * nothing.
	 * 
	 * See java.awt.Component.paint(Graphics)
	 */
	public void paint(Graphics  g)
	{
		if (m_Image != null) {
			g.drawImage(m_Image, 0, 0, m_x, m_y, null, null);
		}
	}
	
	/**
	 * @see java.awt.Component.getPreferredSize()
	 */
	public Dimension getPreferredSize()
	{
		return new Dimension(m_x, m_y);
	}

	/**
	 * @see java.awt.Component.getMinimumSize()
	 */
	public Dimension getMinimumSize()
	{
		return new Dimension(m_x, m_y);
	}
	
	/**
	 * @see java.awt.Component.getMaximumSize()
	 */
	public Dimension getMaximumSize()
	{
		return new Dimension(m_x, m_y);
	}
	
	/**
	 * @see java.awt.Component.isFocusTraversable()
	 */
	public boolean isFocusTraversable()
	{
		return false;
	}
	
	/**
	 * @see java.awt.Component.paramString()
	 */
	public String paramString()
	{
		return new String(
						  this.getName() + 
						  ", resolution=" + String.valueOf(this.m_x) + 
						  "x" + String.valueOf(this.m_y) + 
						  ", image=" + String.valueOf(this.m_Image != null));
	}
}
